function BIFansSZ366(h)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function BIFansSZ366(h)                  MIT, Fall 1999-2005. R. R. Rosales.
%
% Script to study the bifurcation diagram for the equation:
%
% dA/dt = h + epsilon*A - g*A^3 - A^5,
%
% that appears in problem 3.6.6 of the book. This script purpose is to make
% easy to visualize the limit h --> 0 of the bifurcation diagram.    Do not
% use values of h bigger than 0.05,  because then (most of) the interesting
% stuff is not within the figure box.
%
% For a given h > 0, the bifurcation diagrams corresponding to
%
%            g = g_c + 1;
%            g = g_c;
% and        g = g_c - 1;
%
% are drawn, where g_c = -(10/3)*((3/8)*h)^(2/5) is the critical value at
% which the two saddle-node bifurcations (that occur for A > 0) merge and
% disappear;   leaving only an inflection point in the bifurcation curve.
% The merge occurs at:       A_c =          ((3/8)^(1/5))*h^(1/5);
%                      epsilon_c = - ((15/8)*(8/3)^(1/5))*h^(4/5);
%
% The input h is OPTIONAL. If not given, a series illustrating the limit
% h --> 0 will be shown.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Sub-function scripts inside:
%
% @#@1 F_zero_SZans_03_06_06 ... find saddle-node for A < 0.
% @#@2 r_extr_SZans_03_06_06 ... find saddle-nodes and inflections for A >0.
% @#@3 arrowhSZans ............. make arrow-head.
% @#@4 ellipseSZans ............ make ellipses.
%
% --- CHECK. =================================================================
if nargin == 0;
   h = 0.01;
   for n=0:6
      BIFansSZ366(h);
      zoom on
      fprintf('\n BIFansSZ366 message: ')
      fprintf('\n --- Zoom is on, press any key to see next plot.')
      pause
      h = h/2;
   end
   fprintf('\n')
   return
elseif ~(isnumeric(h) & isreal(h) & (h > 0))
   h = 1;
   fprintf('\n BIFansSZ366 message: ')
   fprintf('\n --- Input value of h not allowed. Will use h = 1. \n')
end
% --- ABREVIATIONS. ==========================================================
%
FS = 'FontSize';
FU = 'FontUnits';
FW = 'FontWeight';
LW = 'LineWidth';
MS = 'MarkerSize';
%
% --- ADJUSTABLE PARAMETERS. =================================================
%
tol  = h/1000;% --- tolerance in Newton iterations.
Aran =  1.5;% ------ range in A: - Aran < A < Aran.
rmin = -0.5;% ------ \ range in r is given by: -rmin < r < rmax.
rmax =  1.5;% ------ /
Nb   = 80;% -------- number of points used to plot bifurcation-curves.
Ne   = 25;% -------- number of points used to plot ellipse edges.
% ------------------ location on curves of curve label.
Np   = [36, 44, 52];% --- A > 0 branches.
Nm   = [44, 44, 30];% --- A < 0 branches.
%
% --- COMPUTED PARAMETERS. ===================================================
%
rran = rmax - rmin;
% --- critical values corresponding to given h.
gc = -(10/3)*(((3/8)*h)^(2/5));
Ac =  ((3/8)*h)^(1/5);
rc = -(15/8)*((8/3)^(1/5))*(h^(4/5));
%
% --- select values of g to use: G(1) > G(2) = gc > G(3).
G  = gc + [1 0 -1];
%
% --- curves minimums for A < 0.
[Af, rf] = F_zero_SZans_03_06_06(h*[1 1 1], G, tol);
%
% --- curves inflection points and local extrema for A > 0.
[AM, rM, Am, rm, Ai, ri] = r_extr_SZans_03_06_06(h*[1 1 1], G, tol);
%
% --- OPEN FIGURE WINDOW. ====================================================
%
figure;
set(gcf, 'Name', [' Bifurcation diagram for h = ' num2str(h)], ...
         'NumberTitle', 'off', ...
         'Color', 'w')
%
% --- SET UP AXIS, ETC. ======================================================
%
hold on
hf = gca;
set(hf, LW, 3, FW, 'bold', FS, 12, ...
    'box',        'on', ...
    'Xgrid',      'on', ...
    'Ygrid',      'on', ...
    'TickLength', [0.02 0.02])
axis([rmin rmax -Aran Aran])
% --- critical lines.
plot([rmin rmax], Ac*[1 1], '--c', LW, 2)
plot(rc*[1 1], Aran*[-1 1], '--c', LW, 2)
% --- ylabel
hy = axes('Position', [0.02 0.20 0.07 0.65]);
   axis([0 1 0 1])
   hold on
   arrowhSZans(0.42, 0.90,  pi/2, 0.10, 0.30, 'k')
   plot(0.42*[1, 1], [0.58 0.90], '-k', LW, 4)
   text(0.05, 0.50, '{\bf A^*}', FS, 22)
   plot(0.42*[1, 1], [0.00 0.40], '-k', LW, 4)
   hold off
   axis off
axes(hf)
% --- xlabel
hx = axes('Position', [0.20 0.00 0.65 0.07]);
   axis([0 1 0 1])
   hold on
   POS = get(gcf, 'Position');
   rat = POS(4)/POS(3);
   arrowhSZans(0.90, 0.42, 0, 0.10*rat, 0.30/rat, 'k')
   plot([0.55 0.90], 0.42*[1, 1], '-k', LW, 4)
   text(0.47, 0.50, '{\bf \epsilon}', FS, 22)
   plot([0.00 0.45], 0.42*[1, 1], '-k', LW, 4)
   hold off
   axis off
axes(hf)
% --- title
title('Bifurcation diagrams for dA/dt = h + \epsilon A - gA^3 - A^5.', ...
       FS, 18)
% --- insert
hi = axes('Position', [0.70 0.40 0.27 0.25], ...
          'XTick',  [], ...
          'Xcolor', 'm', ...
          'YTick',  [], ...
          'Ycolor', 'm', ...
          'Box',    'on', ...
           LW,       5);
   axis([0 1 0 1])
   hold on
   text(0.03, 0.81, '{\bf Curve 1:  g = g_c + 1.}', FS, 12)
   text(0.03, 0.63, '{\bf Curve 2:  g = g_c.}',     FS, 12)
   text(0.03, 0.45, '{\bf Curve 3:  g = g_c - 1.}', FS, 12)
   text(0.03, 0.25, '{\bf Critical values: \epsilon_c, A_c}', FS, 12)
   text(0.03, 0.15, '{\bf (cyan lines).}', FS, 12)
   hold off
axes(hf)
%
% --- Plot A < 0 CURVES. =====================================================
%
for n=1:3
    g = G(n);
    % --- stable branches.
    A = -Aran + (Af(n)+Aran)*(0:Nb)/Nb;
    r = (-h./A + g*A.^2 + A.^4);
    plot(r, A,  '-b', LW, 3)
    % --- curve labels: [1 2 3] = g ['>', '=', '<'] gc
    ellipseSZans(r(Nm(n)), A(Nm(n)), 0.020*rran, 0.040*Aran, 'w', Ne);
    text(r(Nm(n))-0.015*rran, A(Nm(n)), ['{\bf ', num2str(n), '}'], FS, 14)
    % --- unstable branches.
    A = Af(n)*[(Nb:-1:1)/Nb, (1/Nb).^[1.2, 1.4, 1.6, 1.8, 2.0, 2.2]];
    r = (-h./A + g*A.^2 + A.^4);
    plot(r, A,  '--r', LW, 3)
end
%
% --- Plot A > 0 CURVES. =====================================================
%
for n=1:3
    g = G(n);
    % --- 1st stable branches.
    A = AM(n)*[(Nb:-1:1)/Nb, (1/Nb).^[1.2, 1.4, 1.6, 1.8, 2.0, 2.2]];
    r = (-h./A + g*A.^2 + A.^4);
    plot(r, A,  '-b', LW, 3)
    % --- unstable branches.
    A = Am(n) + (AM(n)-Am(n))*(0:Nb)/Nb;
    r = (-h./A + g*A.^2 + A.^4);
    plot(r, A,  '--r', LW, 3)
    % --- 2nd stable branches.
    A = Am(n) + (Aran-Am(n))*(0:Nb)/Nb;
    r = (-h./A + g*A.^2 + A.^4);
    plot(r, A,  '-b', LW, 3)
    % --- curve labels: [1 2 3] = g ['>', '=', '<'] gc
    ellipseSZans(r(Np(n)), A(Np(n)), 0.020*rran, 0.040*Aran, 'w', Ne);
    text(r(Np(n))-0.015*rran, A(Np(n)), ['{\bf ', num2str(n), '}'], FS, 14)
end
%
axes(hi)
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Af, rf] = F_zero_SZans_03_06_06(h, g, tol)
% @#@1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate the zero of F(z) =    h - 2*g*z - 4*z^(5/3),  (for z = -A^3 > 0),
% giving the minimum of r(A) = -h/A + g*A^2 + A^4,        (for        A < 0).
%
% Use error tolerance = tol
% Recall      epsilon = r    (epsilon as in problem statement).
%
% Note that h and g can be vectors (same length), in which case A and r are
% vectors (of the same size) with the corresponding answers.
%
% MUST BE h > 0.
% ^^^^^^^^^^^^^^
%               The problem statement explains how the calculation goes.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ERR = 100*tol;
z = max(abs(g).^(1.5), (h/2).^(0.6));
while ERR > tol
   Dz  = (h - 2*g.*z - 4*z.^(5/3))./(2*g + (20/3)*z.^(2/3));
   z   = z + Dz;
   ERR = max(abs(Dz));
end
Af = - z.^(1/3);
rf = -h./Af + g.*(Af.^2) + Af.^4;
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [AM, rM, Am, rm, Ai, ri] = r_extr_SZans_03_06_06(h, g, tol)
% @#@2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate the local maximum and local minimum (for A > 0) of
%                                                ^^^^^^^^^
%                     r(A) = -h/A + g*A^2 + A^4,
%
% when g < g_crit = - (10/3)*((3/8)*h)^(0.6). These are the two zeros of
% ^^^^^^^^^^^^^^^
%                     dr/dA = h/A^2 + 2*g*A + 4*A^3.
%
%       AM, rM .... LOCAL MAXIMUM OF r, SMALLEST ZERO OF dr/dA.
%       Am, rm .... LOCAL MINIMUM OF r, LARGEST  ZERO OF dr/dA.
%
% The INFLECTION POINT (Ai, ri) of r = r(A) is also computed.
%
% WHEN  g >= g_crit, the curve is monotone. Then only the single INFLECTION
%       ^^^^^^^^^^^  POINT of the curve r = r(A) is computed.
%                    Then AM = Am = Ai and rM = rm = ri.
%
% Use error tolerance = tol
% Recall      epsilon = r    (epsilon as in problem statement).
%
% Note that h and g can be vectors (same length), in which case the outputs
% are vectors (of the same size) with the corresponding answers.
%
% MUST BE h > 0.
% ^^^^^^^^^^^^^^
%               The problem statement explains how the calculation goes.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% FUNCTION AND DERIVATIVES ===================================================
%
% r         = r0 = -    h/A   +   g*A^2 +    A^4;
% dr/dA     = r1 = +    h/A^2 + 2*g*A   +  4*A^3;
% d^2r/dA^2 = r2 = -  2*h/A^3 + 2*g     + 12*A^2;
% d^3r/dA^3 = r3 = +  6*h/A^4           + 24*A;
% d^4r/dA^4 = r4 = - 24*h/A^5           + 24;
%
AM = zeros(size(g)); Am = AM; Ai = AM; rM = AM; rm = AM; ri = AM;
N = length(g);
for n=1:N
   % --- Calculate inflection point of r0 = zero of r2. ======================
   %
   % r3 > 0, r2(0) = -inf, r2(inf) = inf ==> r2 has a single zero ......... Ai
   % r4 has a single zero                ==> r2 has a single inflection ... A*
   % r4 < 0 for A < A*                   ==> r2 is concave for A < A*.
   % r4 > 0 for A > A*                   ==> re is convex  for A > A*.
   %
   % Thus a Newton iteration, starting at any A0 <= Ai, will converge to Ai:
   % a) If A* > Ai, iteration will be monotone increasing.
   % b) If A* < Ai, iteration will first be monotone increasing. If it
   %                overshots Ai (if not, convergence will happen in a finite
   %                number of steps), then it will become  monotone decreasing
   %                and converge.
   % A0 = min( ( h/(2*abs(g)) )^(1/3) , (h/12)^(1/5) ) <= Ai, because this
   % value makes the first term in r2 dominant over the other two, so that
   % r2 < 0 for any A < A0. Thus A0 <= Ai.
   %                
   %
   ERR = 100*tol;
   A   = min((h(n)/(2*abs(g(n))))^(1/3),  (h(n)/12)^(1/5));
   while ERR > tol
      DA  = (2*h(n)/A^3 - 2*g(n) - 12*A^2)/(6*h(n)/A^4 + 24*A);
      A   = A + DA;
      ERR = abs(DA);
   end
   Ai(n) = A;
   ri(n) = - h(n)/A + g(n)*A^2 + A^4; 
   %
   if g(n) < -(10/3)*((3/8)*h(n))^(0.4)
      ERR = 100*tol;
      A   = (h(n)/(2*abs(g(n))))^(1/3);
      while ERR > tol
         DA  = (h(n)/A^2 + 2*g(n)*A + 4*A^3)/(2*h(n)/A^3 - 2*g(n) - 12*A^2);
         A   = A + DA;
         ERR = abs(DA);
      end
      AM(n) = A;
      rM(n) = - h(n)/A + g(n)*A^2 + A^4;
      %
      ERR = 100*tol;
      A   = sqrt(abs(g(n))/2);
      while ERR > tol
         DA  = (h(n)/A^2 + 2*g(n)*A + 4*A^3)/(2*h(n)/A^3 - 2*g(n) - 12*A^2);
         A   = A + DA;
         ERR = abs(DA);
      end
      Am(n) = A;
      rm(n) = - h(n)/A + g(n)*A^2 + A^4;
   else
      AM(n) = Ai(n);
      rM(n) = ri(n);
      Am(n) = Ai(n);
      rm(n) = ri(n);
   end
end
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = arrowhSZans(x, y, theta, length, width, color)
% @#@3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function h = arrowhSZans(x, y, theta, length, width, color)
% ____________________________________________________________________________
% Places arrowhead (isosceles triangle of base = width and height = length) in
% current figure. The OTHER input variables are:
% -- (x, y) ....... center-point of the base of the arrowhead.
% -- theta ........ direction of arrowhead (in radians).
% -- color ........ string variable with the color (e.g.: color = 'r';).
%                   The arrowhead is filled solid with the color.
% OPTIONAL: color can be omitted, in which case black is used.
%           width and color can be omitted, in which case width = length/2.
% -- h ............ handle for the arrowhead.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
TRUE = 1;
if ~ishold; TRUE = 0; hold on;   end % --- Record/adjust hold status.
if nargin < 6; color = 'k';      end % --- Default color.
if nargin < 5; width = length/2; end % --- Default width.
diA = [cos(theta), sin(theta)];% --------- Direction along arrowhead.
per = [-diA(2), diA(1)];% ---------------- Direction normal to arrowhead.
tip = [x, y] + length*diA;% -------------- Tip of arrowhead.
blf = [x, y] - width*per/2;% ------------- Left vertex along base.
brt = [x, y] + width*per/2;% ------------- Right vertex along base.
xx  = [tip(1), blf(1), brt(1)];% --------- x-coordinate arrow boundary.
yy  = [tip(2), blf(2), brt(2)];% --------- y-coordinate arrow boundary.
h = fill(xx, yy, color);% ---------------- Make arrow.
if nargout == 0;   clear h;      end % --- Clean up h if no output required.
if TRUE == 0;     hold off;      end % ------ Return hold to original.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ellipseSZans(x, y, r1, r2, color, N, phi)
% @#@4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function ellipseSZans(x, y, r1, r2, color, N, phi)
% ____________________________________________________________________________
% Places a (filled with color) ellipse of radii r1 and r2, centered at (x, y),
% in the current figure. The input variables are:
% -- (x, y) ....... center of ellipse.
% -- r1 ........... horizontal radius of ellipse.
% -- r2 ........... vertical radius of ellipse.
% -- color ........ string variable with the color (e.g.: color='r').
% -- N ............ number of points used to draw ellipse boundary.
% -- phi .......... phase, to allow rotation of polygon when N is small.
% OPTIONAL: phi can be omitted, in which case phi = 0.
% OPTIONAL: N can be omitted, in which case N = 10.
% OPTIONAL: color and N can be omitted, in which case black is used for color.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 7;  phi = 0;     end
if nargin < 6;  N = 10;      end
if nargin < 5;  color = 'k'; end
xx = x + r1*cos(phi+2*pi*(0:N)/N);
yy = y + r2*sin(phi+2*pi*(0:N)/N);
TRUE = 1;
if ~ishold; TRUE = 0; hold on;   end % --- Record/adjust hold status.
fill(xx, yy, color);
plot(xx, yy, '-', 'Color', color, 'LineWidth', 0.5)
if TRUE == 0;     hold off;      end % --- Return hold to original
if nargout == 0;   clear h;      end % --- Clean up h if no output required.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  10-07-1999.
% Version 02 by Rodolfo R. Rosales  11-12-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% EOF
