function BIFstrstb(choice)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function BIFstrstb(choice)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).    MIT, Fall 1999-2005. R. R. Rosales.
%
% Structural stability for bifurcations of critical points:
% Script plots the surfaces of equillibrium solutions  for the Normal Forms in
% bifurcations of critical points.      That is:   it plots the 2-D surface of
% equillibrium solutions,  as functions of the bifurcation parameter (r) and a
% perturbation parameter (h).  For each surface 3 figures are made, displaying
% three selected points of view  (to make the structure of the surface and its
% singularities evident) .... rotate3d can be used to see the surface from any
% other angle (or view, in MatLab versions < 5.x).
%
% Has the options:
% choice = 1 -- Transcritical bifurcation.  Normal form: dx/dt = h + r*x - x^2.
% choice = 2 -- Soft pitchfork bifurcation. Normal form: dx/dt = h + r*x - x^3.
%               (Note that the hard pitchfork bifurcation has the same surface
%               [via x --> -x and r --> -r], the change being in the stability
%               of the solutions, which is the reverse of the soft pitchfork).
%
% The parameter choice is optional in the function call. If none is given, the
% parameter will be requested via screen input.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 1
   fprintf('\n Enter choice = 1 for TRANSCRITICAL bifurcation.')
   fprintf('\n Enter choice = 2 for PITCHFORK bifurcation. \n')
   choice = input(' ..... choice = ');
end
%
choice = floor(choice);
if (choice < 1); choice = 1; end
if (choice > 2); choice = 2; end
%
FN  = 'FontName';
FS  = 'FontSize';
HNB = 'HelveticaNarrowBold';
%
Nr = 160;    % Number of points in r for the plot.
Rr = 10;     % Range of r, i.e.: -Rr < r < Rr.
Nx = 100;    % Number of points in x for the plot.
dr = 2*Rr/Nr;
rr = -Rr:dr:(Rr - dr);
rr = rr';
r  = rr*ones(1,Nx);    %%%%% Note: r_(i,j) = r_i = -Rr + (i-1)*dr
%
if choice == 1
%  %
%  % --------- TRANSCRITICAL BIFURCATION HERE ********************************
%  %
%  % Surface of equillibrium solutions -- as a functions of r and h -- for the
%  % Transcritical Bifurcation Structural Stability Analysis Normal Form:
%  %
%  % dx/dt = h + r*x - x^2 
%  %
   Rx = 15;    % Range of x, i.e.: -Rx < x < Rx.
   dx = 2*Rx/Nx; 
   xx = -Rx:dx:(Rx - dx);
   x  = ones(Nr,1)*xx;    %%%%% Note: x_(i,j) = x_j = -Rx + (j-1)*dx
   h  = -r.*x + x.^2;
   Rh = 0.25*Rr^2;
   I  = find(abs(h) > Rh);
   J  = find(abs(h) <= Rh);
   x1 = min(min(x(J)));
   x2 = max(max(x(J)));
   C  = min(max(x, x1), x2);
   x(I) = Inf*I;
%  %
   f1 = figure;
   F1pos = get(f1, 'Position');
   shx = F1pos(3)/4;
   shy = F1pos(4)/4;
   F2pos = [F1pos(1) - shx, F1pos(2) - shy, F1pos(3), F1pos(4)];
   F3pos = [F1pos(1) + shx, F1pos(2) - 2*shy, F1pos(3), F1pos(4)];
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh x1 x2])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Transcritical Bifurcation', FN, HNB, FS, 14);
   view(40, 25)
   grid on
%  %
   f2 = figure;
   set(f2, 'Position', F2pos);
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh x1 x2])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Transcritical Bifurcation', FN, HNB, FS, 14);
   view(20, 40)
   grid on
%  %
   f3 = figure;
   set(f3, 'Position', F3pos);
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh x1 x2])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Transcritical Bifurcation', FN, HNB, FS, 14);
   view(0, 90)
   grid on
%  %
elseif choice == 2
%  %
%  % --------- PITCHFORK BIFURCATION HERE ************************************
%  %
%  % Surface of equillibrium solutions -- as a functions of r and h -- for the
%  % Soft PitchFork Bifurcation Structural Stability Analysis Normal Form:
%  % 
%  %               dx/dt = h + r*x - x^3 
%  %
   Rx =  3.7;  % Range of x, i.e.: -Rx < x < Rx.
   dx = 2*Rx/Nx;
   xx = -Rx:dx:(Rx - dx);
   x  = ones(Nr,1)*xx;    %%%%% Note: x_(i,j) = x_j = -Rx + (j-1)*dx
   h  = -r.*x + x.^3;
   Rh = 2*(Rr/3)^1.5;
   I  = find(abs(h) > Rh);
   J  = find(abs(h) <= Rh);
   x1 = min(min(x(J)));
   x2 = max(max(x(J)));
   C  = min(max(x, x1), x2);
   x(I) = Inf*I;
   %
   f1 = figure;
   F1pos = get(f1, 'Position');
   shx = F1pos(3)/4;
   shy = F1pos(4)/4;
   F2pos = [F1pos(1) - shx, F1pos(2) - shy, F1pos(3), F1pos(4)];
   F3pos = [F1pos(1) + shx, F1pos(2) - 2*shy, F1pos(3), F1pos(4)];  
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh -Rx Rx])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Soft Pitchfork Bifurcation', FN, HNB, FS, 14);
   view(-30, 20)
   grid on
   %
   f2 = figure;
   set(f2, 'Position', F2pos);
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh -Rx Rx])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Soft Pitchfork Bifurcation', FN, HNB, FS, 14);
   view(40, 40)
   grid on
   %
   f3 = figure;
   set(f3, 'Position', F3pos);
   whitebg('k')
   colormap hsv
   surf(r, h, x, C)
   shading interp
   colorbar
   axis([-Rr Rr -Rh Rh -Rx Rx])
   xlabel('r', FN, HNB, FS, 20);
   ylabel('h', FN, HNB, FS, 20);
   zlabel('x', FN, HNB, FS, 20);
   title('Perturbed Soft Pitchfork Bifurcation', FN, HNB, FS, 14);
   view(0, 90)
   grid on
end
%
fprintf('\n ******************************************** \n')
disp([' THREE FIGURES WITH DIFFERENT VIEWS PRODUCED.'; ...
      ' To see other views use rotate3d or view.    '])
fprintf('\n ******************************************** \n')

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  10-07-1999.
% Version 02 by Rodolfo R. Rosales  11-12-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% End of File

