function y_dot = PHPLfunct(t, y)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 18.385 (Nonlinear Dynamical Systems).    MIT, Fall 1999-2004. R. R. Rosales.
% ____________________________________________________________________________
% y_dot = PHPLfunct(t, y) calculates f and g for the nonlinear oscilator:
% 
%             du/dt = f(u,v)     and     dv/dt = g(u,v),
%
% where y = [u, v] and y_dot = [du/dt, dv/dt].  Notice that, in fact, t plays
% no role here   (it is an argument in the function because ode45 require the
% argument there).
%
% If glo_dir = -1 (a global variable) then the negatives of the functions are
% evaluated.  This is used to do forward and backward integrations of the ode.
%
% The OPTIONS (picked via the global variable glo_option) are:
%
% WRITE YOUR OWN ........... (#00)  du/dt = f(u,v),    dv/dt = g(u,v).
% Saddle and 2 centers ..... (#01)  du/dt = v,         dv/dt = u*(1-u^2).
% Limit cycle .............. (#02)  du/dt = v,         dv/dt = -u+(1-u^2)*v.
% Pendulum ................. (#03)  du/dt = v,         dv/dt = -sin(u).
% Harmonic osc. (center) ... (#04)  du/dt = v,         dv/dt = -u.
% Linear stable spiral ..... (#05)  du/dt = v-0.1*u,   dv/dt = -u - 0.1*v.
% Linear saddle ............ (#06)  du/dt = u,         dv/dy = -v.
% Center --> Spiral by \ ... (#07)  du/dt = -v - u*R,  dv/dt =  u - v*R,
% nonlinearity         /               R = (u^2+v^2).
% Two eyed monster ......... (#08)  du/dt = v + v^2,
%                                   dv/dt = -0.5*u + 0.2*v - u*v + 1.2*v^2.
% Dipole fixed point ....... (#09)  du/dt = 2*u*v,     dv/dt = v^2 - u^2.
% Lotka-Volterra \ ......... (#10)  du/dt =  (1-0.4*v)*u, (u = prey),
% predator-prey  /                  dv/dt = -(1-0.6*u)*v, (v = predator).
% Lotka-Volterra \ ......... (#11)  du/dt = u*(3-u-2*v), (u = "rabbits"),
% competition    /                  dv/dt = v*(2-u-v),   (v = "sheep").
% Parametric Limit\ ........ (#12)  dr/dt = r*(1-r^2)+mu*r*cos(theta),
% Cycle           /                 dtheta/dt = 1.
% Damped--driven \ ......... (#13)  du/dt = v
% pendulum       /                  dv/dt = I - alpha*v - sin(u)
%
% SPECIAL NOTES:
% (#12) ... The variable mu is requested via screen input from the user.
% (#13) ... The variables alpha and I have defaults:  alpha = 0.5 and I = 0.8.
%           The defaults can be changed by defining global variables glob_Torq
%           and glob_alph. Then I will take the value glob_Torq and alpha will
%           take the value glob_alpha.
%           Note that changing the defaults by a lot may make the default plot
%           axes useless. These axes are good for I = O(1) and alpha = 0(1).
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ************************************** Global Variables *******************
% glo_option .................. selects example.
% TEXTT ....................... title of figure.
% [umin, umax, vmin, vmax] .... figure bounds.
% Rmax2 ....................... switch radius squared: turn-off nonlinearity
%                               to avoid finite time blow-up in solution.
% glo_dir ..................... determines direction of time integration.
% TEXTF, TEXTG ................ string variables with user functions f and g.
% glob_mu ..................... parameter in equation.
%
global glo_option TEXTT umin umax vmin vmax Rmax2 glo_dir TEXF TEXG glob_mu
%
y_dot = y;
% ----- Modification to input functions without having to type them.
% TEXF = 'real(i*(u+i*v)*(1-(u+i*v)^3))';
% TEXG = 'imag(i*(u+i*v)*(1-(u+i*v)^3))';
% umin = -4;
% umax =  4;
% vmin = -4;
% vmax =  4;
%    set(gca, 'XTick', [-0.75, -0.50, -0.25, 0.00, 0.25, 0.50, 0.75], ...
%             'YTick', [-0.75, -0.50, -0.25, 0.00, 0.25, 0.50, 0.75])
% ----- END OF MODIFICATION.
%
% ************************************** WRITE YOUR OWN *********************
% Note: in this one the actual function is provided via global text variables
%       from somewhere else (user input).    No figure bounds are created, as
%       these are also requested from the user elsewhere.
%
if glo_option == 0
   u = y(1); v = y(2);
   y_dot(1) = eval(TEXF);
   y_dot(2) = eval(TEXG);
%
% ************************************** EXAMPLE #1 *************************
% Saddle and two centers.
%
elseif glo_option == 1
   y_dot(1) = y(2);
   y_dot(2) = y(1)*(1-y(1)^2);
   TEXTT = 'Equation: du/dt = v,  dv/dt = u*(1-u^2).'; %%%<< TITLE.
   umin = -2; umax = 2; vmin = -2; vmax = 2;           %%%<< Figure bounds.
%
% ************************************** EXAMPLE #2 *************************
% Limit cycle.
%
elseif glo_option == 2
   y_dot(1) = y(2);
   y_dot(2) = -y(1) + y(2)*(1-y(1)^2);
   TEXTT = 'Equation: du/dt = v,  dv/dt = -u + (1-u^2)*v.';
   umin = -4; umax = 4; vmin = -4; vmax = 4;
%
% ************************************** EXAMPLE #3 *************************
% Pendulum.
%
elseif glo_option == 3
   y_dot(1) = y(2);
   y_dot(2) = - sin(y(1));
   TEXTT = 'Equation: du/dt = v,  dv/dt = - sin(u).';
   umin = -4; umax = 4; vmin = -4; vmax = 4;
%
% ************************************** EXAMPLE #4 *************************
% Harmonic Oscillator (center).
%
elseif glo_option == 4
   y_dot(1) = y(2);
   y_dot(2) = - y(1);
   TEXTT = 'Equation: du/dt = v,  dv/dt = - u.';
   umin = -4; umax = 4; vmin = -4; vmax = 4;
%
% ************************************** EXAMPLE #5 *************************
% Linear stable spiral.
%
elseif glo_option == 5
   y_dot(1) = y(2) - 0.1*y(1);
   y_dot(2) = - y(1) - 0.1*y(2);
   TEXTT = 'Equation: du/dt = v-0.1*u,  dv/dt = -u-0.1*v.';
   umin = -4; umax = 4; vmin = -4; vmax = 4;
%
% ************************************** EXAMPLE #6 *************************
% Linear saddle.
%
elseif glo_option == 6
   y_dot(1) = y(1);
   y_dot(2) = - y(2);
   TEXTT = 'Equation: du/dt = u,  dv/dt = - v.';
   umin = -4; umax = 4; vmin = -4; vmax = 4;
%
% ************************************** EXAMPLE #7 *************************
% Center into spiral by nonlinearity.
%
elseif glo_option == 7
   R = y(1)^2 + y(2)^2;
   y_dot(1) = - y(2) - 0.1*y(1)*R;
   y_dot(2) =   y(1) - 0.1*y(2)*R;
   TEXTT = 'du/dt=-v-0.1*u*R,  dv/dt=u-0.1*v*R, R=u^2+v^2.';
   umin = -1.5; umax = 1.5; vmin = -1.5; vmax = 1.5;
%
% ************************************** EXAMPLE #8 *************************
% Two eyed monster.
%
elseif glo_option == 8
   y_dot(1) = y(2) + y(2)^2;
   y_dot(2) = -0.5*y(1) + 0.2*y(2) - y(1)*y(2) + 1.2*y(2)^2;
   TEXTT = 'Two eyed monster.';
   umin = -6; umax = 4; vmin = -4; vmax = 3;
%
% ************************************** EXAMPLE #9 *************************
% Dipole fixed point.
%
elseif glo_option == 9
   y_dot(1) = 2*y(1)*y(2);
   y_dot(2) = y(2)^2 - y(1)^2;
   TEXTT = 'du/dt = 2*u*v,  dv/dt = v^2 - u^2.';
   umin = -2.5; umax = 2.5; vmin = -2.5; vmax = 2.5;
%
% ************************************** EXAMPLE #10 ***********************
% Lotka-Volterra predator-prey (u = prey and v = predator).
%
elseif glo_option == 10
   y_dot(1) =   (1 - 0.4*y(2))*y(1);
   y_dot(2) = - (1 - 0.6*y(1))*y(2);
   TEXTT = 'du/dt = (1-0.4*v)*u,  dv/dt = -(1-0.6*u)*v.';
   umin = 0; umax = 5; vmin = 0; vmax = 5; 
%
% ************************************** EXAMPLE #11 ***********************
% Lotka-Volterra competition (u = "rabbits" and v = "sheep").
%
elseif glo_option == 11
   y_dot(1) = (3 - y(1) - 2*y(2))*y(1);
   y_dot(2) = (2 - y(1) -   y(2))*y(2);
   TEXTT = 'du/dt = u*(3-u-2*v),  dv/dt = v*(2-u-v).';
   umin = 0; umax = 3.5; vmin = 0; vmax = 3.5;   
%
% ************************************** EXAMPLE #12 ***********************
% Parametric Limit Cycle.
%
elseif glo_option == 12
   r  = norm(y);
   R  = 1 - r^2 + glob_mu*y(1)/(1e-6 + r);
   y_dot(1) = R*y(1)-y(2);
   y_dot(2) = R*y(2)+y(1);
   TEXTT = ['dr/dt = r*(1-r^2)+', num2str(glob_mu), ...
            '*r*cos(\theta),   d\theta/dt = 1.'];
   umin = -1.25; umax = 3.25; vmin = -1.25; vmax = 3.25; 
%
% ************************************** EXAMPLE #13 ***********************
%
elseif glo_option == 13
   global glob_Torq glob_alph
   if isempty(glob_Torq); glob_Torq = 0.8; end
   if isempty(glob_alph); glob_alph = 0.5; end
   y_dot(1) = y(2);
   y_dot(2) = glob_Torq - glob_alph*y(2) - sin(y(1));
   TEXTT = ['Damped -- driven pendulum for:', ...
            '  \alpha = ', num2str(glob_alph), ...
            '  and  I = ', num2str(glob_Torq)];
   umin = 0; umax = 2*pi; vmin = -2; vmax = 4;
%
%
% ************************************** EXAMPLE #14 ***********************
%
%
%
% ************************************** EXAMPLE #15 ***********************
%
%
%
% ************************************** EXAMPLE #16 ***********************
%
end
if glo_dir == -1
   y_dot = - y_dot;
end
%
% ***** TURN-OFF the NONLINEARITY ******************************************
%
% Note that p(x) = (2*x-1)*(x-2)^2 satisfies: p(1)  = 1; p(2)  = 0;
%                                             p'(1) = 0; p'(2) = 0;
% so that it can be used to "turn-off" the equation "smoothly".
%
y2 = sum(y.^2);
if y2 > Rmax2
   p = y2/Rmax2;
   y_dot = ((2*p-1)*(p-2)^2)*y_dot;
   if p > 2
      y_dot = 0;
   end
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  10-24-1999.
% Version 02 by Rodolfo R. Rosales  10-17-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF

