function PHPLplot(f, g, U, V, ejes, t_f, BF, COL, LS, LWn, FAR, tol)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PHPLplot(f, g, U, V, ejes, t_f, BF, COL, LS, LWn, FAR, tol)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).    MIT, Fall 1999-2004. R. R. Rosales.
%
% Script to illustrates the Phase Plane behavior of the nonlinear oscillator:
%
%                du/dt = f(u,v)    and   dv/dt = g(u,v),
%
% by computing (and plotting) at once several orbits.
%
% The purpose of this script is: to allow the production of very good figures
% with phase plane portraits. It is as "user-friendly" as possible, given the
% limitations that the need to have fine control over multiple figure details
% impose (these are crucial in getting xtra-good pictures).
%
% INPUTS:
%        1) f & g are string variables, with the formulas for f & g in the ODE
%           to be solved. Can be written using any valid MatLab operation.
%
%        2) U and V are arrays with the pairs of starting points (u = u(0) and
%           v = v(0)) for the integrations in the phase plane.
%
%        3) ejes = [umin umax vmin vmax] is an array giving the bounds on the
%                  plotting region in the phase plane.
%
%        4) The integration is done for:  -t_f < t < t_f  if  BF = 1 and
%                                           0 < t < t_f   if  BF = 0.
%           t_f is a column array giving the time to integrate for each orbit.
%           If t_f = [], then t_f = 25 for all the orbits.
%
%        5) COL is a column array giving the colors that each orbit  (as given
%           by (u(0), v(0)) will have.   Each row assigns a color to an orbit.
%           Must have the same number of entries as U and V.       It can have
%           either of the two following forms:
%              COL = ['r'; 'b'; ...]  ------> colors given by letter symbols.
%              COL = [1 0 0; 0 0 1; ...] ---> colors given in RGB fashion.            
%           If COL = [], then all curves will be plotted in BLACK.
%
%        6) LS is a column array giving the line style for each orbit.    Same
%           idea as COL. Any of the styles allowed by plot work. Example:
%              LS = [' -';  '--';  '-.';  ' :';  ' x'; ...].
%           If LS = [], then all curves will be plotted in SOLID LINE '-'.
%
%        7) LWn is a column array giving the LineWidth for each orbit.    Same
%           idea as COL. Example:
%              LWn = [ 2; 1; 3; 6; ...]
%           If LWn = [], then all curves will be plotted with LineWidth = 2.
%
%        8) FAR is a column array determining the length of the arrows on each
%           orbits (that point the direction of the flow).  Each arrow is:
%                   length = (diagonal of plotting rectangle)/FAR.
%           If FAR = [], then all arrows have FAR = 25.
%
%        9) tol is the tolerance used by the solver (ode45).   It is optional,
%           with DEFAULT = 1e-7.
%
% Plotting is done at the end of the computation of each orbit.
%
% See PHPLreadme for other Phase Plane scripts.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
FBG = 'w';% --- Background used in the figure.
%
% ************************************** Global Variables ******************
% glo_dir ..................... determines direction of time integration.
%
global glo_f  glo_g  glo_dir  Rmax2
%
glo_f = f; glo_g = g;
%
% ***************************** Figure out MatLab version number. **********
%
Mat_ver = version;
Mat_ver =str2num(Mat_ver(1));
if (Mat_ver < 4) | (Mat_ver > 5)
   fprintf('\n ')
   disp([' WARNING: this script may not work properly with '; ...
         '          MatLab versions other than 4 or 5.     ']);
end
%
% ************ Set up parameters, etc. *************************************
%
if nargin == 11
   tol = 1e-7;
end
%
if length(t_f) == 0
   t_f = 25*ones(length(U), 1);
end
%
if length(FAR) == 0
   FAR = 25*ones(length(U), 1);
end
%
if length(COL) == 0
   COL = ones(length(U), 1)*[0 0 0];
end
%
if length(LWn) == 0
   LWn = 2*ones(length(U), 1);
end
%
if length(LS) == 0
   for n=1:length(U);
      LS = [LS; '-'];
   end
end
%
if Mat_ver >= 5
   StrRelTol = 'RelTol'; StrAbsTol = 'AbsTol';
   eval('options=odeset(StrRelTol, 10*tol, StrAbsTol, tol*[0.01 0.01]);');
end
%
umin = ejes(1); umax = ejes(2); vmin = ejes(3); vmax = ejes(4);
arrow = (sqrt((umax-umin)^2 + (vmax-vmin)^2))./FAR; % Length of arrow to use.
% Rmax2 ....................... switch radius squared: turn-off nonlinearity
%                               to avoid finite time blow-up in solution.
Rmax2 = 1.2*(max(umin^2, umax^2) + max(vmin^2, vmax^2));
TEXTT = ['du/dt = ', f, ' and dv/dt = ', g];% --- Title of figure.
%
% ***************************** Abreviations for titles, etc. **************
%
FN  = 'FontName';
FU  = 'FontUnits';
FW  = 'FontWeight';
FS  = 'FontSize';
HNB = 'HelveticaNarrowBold';
LW  = 'LineWidth';
%
% **** Now code starts. ****************************************************
%
% +++ Set up figure ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure
whitebg(FBG)
axis(ejes)
hold on
ha = gca;
if Mat_ver < 5
   set(ha, FW, 'demi', LW, 2, FS, 16)
elseif Mat_ver > 4
   set(ha, FU, 'points', FW, 'demi', LW, 2, FS, 16)
end
grid on
title(TEXTT, FN, HNB, FS, 14)
xlabel('u(t)', FN, HNB, FS, 16) 
ylabel('v(t)', FN, HNB, FS, 16, 'Rotation', 0)
%
for n=1:length(U);
    glo_dir = 1;
    t_0 = 0; y_0 = [U(n), V(n)];
    if Mat_ver >= 5
       [T, Y] = ode45('PHPLplotFUN', [t_0, t_f(n)], y_0, options);
    else
       [T, Y] = ode45('PHPLplotFUN', t_0, t_f(n), y_0, tol);
    end
    %
    % --- MAKE NOW A SMALL ARROW IN THE DIRECTION OF THE FLOW.
    %
    u0 = y_0(1);
    v0 = y_0(2);
    y_1 = PHPLplotFUN(t_0, y_0);
    Du = y_1(1);
    Dv = y_1(2);
    step = sqrt(Du^2+Dv^2);
       u1 = u0 + (arrow(n)/step)*Du;
       v1 = v0 + (arrow(n)/step)*Dv;
       u2 = u0 + (arrow(n)/step)*Dv/4;
       v2 = v0 - (arrow(n)/step)*Du/4;
       u3 = u0 - (arrow(n)/step)*Dv/4;
       v3 = v0 + (arrow(n)/step)*Du/4;
    fill([u1, u2, u3], [v1, v2, v3], COL(n, :))
    plot(Y(:,1), Y(:,2), LS(n, :), 'Color', COL(n, :), LW, LWn(n))
    %
    if BF == 1
       glo_dir = -1;
       if Mat_ver >= 5
          [T, Y] = ode45('PHPLplotFUN', [t_0, t_f(n)], y_0, options);
       else
          [T, Y] = ode45('PHPLplotFUN', t_0, t_f(n), y_0, tol);
       end
       plot(Y(:,1), Y(:,2), LS(n, :), 'Color', COL(n, :), LW, LWn(n))
    end
end
hold off
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version a by Rodolfo R. Rosales  10-24-1999.
% Version b by Rodolfo R. Rosales  10-18-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF

