function y_dot = PHPLplot_v2(f, g, DATA)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PHPLplot_v2(f, g, DATA)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).    MIT, Fall 2000-2004. R. R. Rosales.
%
% Script to illustrate the Phase Plane behavior of the nonlinear oscillator:
%
%                du/dt = f(u,v)    and   dv/dt = g(u,v),
%
% by computing (and plotting) at once several orbits, with initial conditions:
%
% u(0) = u10, u20, ...., uN0 \ Individual control of the orbit graphic style
% v(0) = v10, v20, ...., vN0 / is allowed (color, linewidth, etc.).
%
% Plotting is done at the end of the computation of each orbit.
%
% The purpose of this script is: to allow the production of very good figures
% with phase plane portraits. It is as "user-friendly" as possible, given the
% limitations that the need to have fine control over multiple figure details
% impose (these are crucial in getting xtra-good pictures). Since most of the
% parameters are optional, the code can also be used at a rather simple level
% --- while still getting reasonable pictures. This second version allows any
% of the optional parameters to be skipped, unless the prior version where to
% include a parameter, all prior ones had to be included too.
%
% Version 2 of the PHPLplot script (REQUIRES MatLab VERSION higher than 5.0).
% Changes: (a) Integrated into one file,  rather than the two files
%              (PHPLplot.m & PHPLplotFUN.m) the first version needs.
%          (b) It uses a structure to input most of the variables. This allows
%              more flexibility,   such as the ability to skip optional inputs
%              without worrying about order.  All the options available in the
%              prior version are still available, but the input variables have
%              been given more representative names.
%          (c) The extra input DATA.fun (see below) which allows this script
%              to be buried into another one.
%          (d) The extra, optional, inputs (see below):
%
%              DATA.FigName
%              DATA.FontName
%              DATA.title
%              DATA.titleFsz
%              DATA.xlabel
%              DATA.xlablFsz
%              DATA.ylabel
%              DATA.ylablFsz
%              DATA.rotation
%              DATA.axesLineWidth
%              DATA.axesFontWeight
%              DATA.axesFontSize
%
%              These allow extra-fine control of how the figure will look.
%
% See PHPLreadme for other Phase Plane scripts.
%
% The inputs are as follows (see end of file for a compact listing).
% WARNING: this script does almost no internal error checking to detect
%          parameters improperly defined.  It is left up to the user to
%          make sure the inputs are sensible.
%
% f ....... String variable, with the formula for f in the ODE to be solved.
%           Can be written using any valid MatLab operation.
% g ....... String variable, with the formula for g in the ODE to be solved.
%           Can be written using any valid MatLab operation.
% DATA .... A 1x1 structure with the fields:
%           (Below: N = number of orbits computed).
% DATA.U ........ Row array with the initial data for u.
%                 DATA.U = [u10, u20, ... un0, ... uN0].
% DATA.V ........ Row array with the initial data for v:
%                 DATA.U = [v10, v20, ... vn0, ... vN0].
% DATA.axis ..... Row array, giving the bounds on the plotting region in the
%                 phase plane. DATA.axis = [umin umax vmin vmax].
%           The parameters below are all OPTIONAL. ==========================
% DATA.t_f ...... Column array giving the time to integrate for each orbit.
%                 DEFAULT: DATA.t_f(n) = 25 for all n=1:N.
% DATA.BackForw . DATA.BackForw = 1: Integration is done for: -t_f < t < t_f.
%                 DATA.BackForw = 0: Integration is done for:    0 < t < t_f.
%                 DEFAULT: DATA.BackForw = 1.
% DATA.COLORS ... Array giving the colors that each orbit will have. Each row
%                 assigns a color to an orbit (must have N rows). It can have
%                 either of the two following forms:
%                 1) Colors given by letter symbols:
%                    DATA.COLORS = ['r'; 'b'; ...];
%                 2) Colors given in RGB fashion:
%                    DATA.COLORS = [1 0 0; 0 0 1; ...];
%                 DEFAULT: all curves plotted in BLACK.
% DATA.LineStyle  Array with the line styles for each orbit. Each row assigns
%                 a style to an orbit (must have N rows).   Any of the styles
%                 allowed by plot work. Example:
%                 DATA.LineStyle = [' -';  '--';  '-.';  ' :';  ' x'; ... ].
%                 (Note the spaces added to make all lines the same length).
%                 DEFAULT: all curves plotted in SOLID LINE '-'.
% DATA.LineWidth  Column array giving the LineWidth for each orbit.  Example:
%                 DATA.LineWidth = [ 2; 1; 3; 6; ...].
%                 DEFAULT: all curves plotted with LineWidth = 2.
% DATA.FracArrow  Column array determining the length of the direction arrows
%                 on each orbit.  Each arrow is:
%                   length = (diagonal of plotting rectangle)/FracArrow.
%                 DEFAULT: DATA.FracArrow(n) = 25 for n=1:N.
% DATA.tol ...... Tolerance used by the solver (ode45).
%                 DEFAULT: DATA.tol = 1e-7.
% DATA.fun ...... String with the name of the script. This may sound foolish,
%                 but it allows:
%                 a) To change the name of the file with the script having to
%                    worry only about inputing the new file name here to make
%                    sure it works  (this is needed because this script calls
%                    itself --- via ode45 --- when solving the equation).
%                 b) It allows to have the right hand side in the ode done by
%                    some other script (ignoring f and g).  This other script
%                    must have the standard form ode45 requires.
%                 DEFAULT: DATA.fun = 'PHPLplot_v2'.
% DATA.FigName .. String with the name  in the menu bar of the figure window.
%                 DEFAULT: 'PHPL_plot_v2 (Fig. #)'  (# is the figure handle).
% DATA.FontName . FontName for title, xlabel and y label of the figure.  Some
%                 examples: 'Helvetica', 'Geneva', 'Monaco'.
%                 DEFAULT: MatLab's default (depends on OS).
% DATA.title .... String with the figure title.
%                 DEFAULT: '{\bf du/dt = f and dv/dt = g}', where f and g are
%                 the input strings.
% DATA.titleFsz . FontSize for title. ................ DEFAULT: 14.
% DATA.xlabel ... String with the figure xlabel. ..... DEFAULT: '{\bf u(t)}'.
% DATA.xlablFsz . FontSize for xlabel. ............... DEFAULT: 16.
% DATA.ylabel ... String with the figure ylabel. ..... DEFAULT: '{\bf v(t)}'.
% DATA.ylablFsz . FontSize for ylabel. ............... DEFAULT: 16.
% DATA.rotation . Orientation of ylabel. In degrees,
%                 counter-clock-wise from horizontal.  DEFAULT:  0.
% DATA.axesLineWidth ................................. DEFAULT:  2.
% DATA.axesFontWeight ................................ DEFAULT: 'demi'.
%                 Options: light, normal, demi, bold.
% DATA.axesFontSize .................................. DEFAULT: 16.
%                 These last three affect the axis lines, tick-marks
%                 and the tick-marks labels.
%
% Note: ode45 calls this script in the form y_dot = PHPLplot_v2(t, y) (unless
%       DATA.fun is provided).  Then this script provides the function needed
%       to solve the equation.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargout == 1;
   y_dot = PHPLplotFUNv2(f, g);
   return
end
%
FBG = 'w';% --- Background used in the figure.
%
% ************************************** Global Variables ********************
% glo_dir ..................... determines direction of time integration.
% Rmax2 ....................... radius at which ODE right hand side starts
%                               to be "turned off", to avoid blow up.
%
global glo_f  glo_g  glo_dir  Rmax2
%
glo_f = f; glo_g = g;
%
% ***************************** Figure out MatLab version number. ************
%
Mat_ver = version;
Mat_ver =str2num(Mat_ver(1));
if (Mat_ver ~= 5)
   fprintf('\n ')
   disp([' WARNING: this script may not work properly with '; ...
         '          MatLab versions other than 5.          ']);
end
%
% ************ Set up parameters, etc. ***************************************
%
N = length(DATA.U);
%
U    = DATA.U;
V    = DATA.V;
ejes = DATA.axis;
%
if isfield(DATA, 't_f')
            t_f = DATA.t_f;
else;       t_f = 25*ones(N, 1);         end
% ---------------------------------------- %
if isfield(DATA, 'BackForw');
             BF = DATA.BackForw;
else;        BF = 1;                     end
% ---------------------------------------- %
if isfield(DATA, 'COLORS')
            COL = DATA.COLORS;
else;       COL = zeros(N, 3);           end
% ---------------------------------------- %
if isfield(DATA, 'LineStyle')
             LS = DATA.LineStyle;
else;        LS = char(45*ones(N, 1));   end
% ---------------------------------------- %
if isfield(DATA, 'LineWidth')
            LWn = DATA.LineWidth;
else;       LWn = 2*ones(N, 1);          end
% ---------------------------------------- %
if isfield(DATA, 'FracArrow')
            FAR = DATA.FracArrow;
else;       FAR = 25*ones(N, 1);         end
% ---------------------------------------- %
if isfield(DATA, 'tol')
            tol = DATA.tol;
else;       tol = 1e-7;                  end
% ---------------------------------------- %
if isfield(DATA, 'fun')
            fun = DATA.fun;
else;       fun = 'PHPLplot_v2';         end
% ---------------------------------------- %
% DATA.FigName (see when figure is opened).
% ---------------------------------------- %
if isfield(DATA, 'FontName')
             FN = DATA.FontName;
else;        FN = [];                    end
% ---------------------------------------- %
if isfield(DATA, 'title')
       FigTitle = DATA.title;
else;  FigTitle = ['{\bf du/dt = ', f, ' and dv/dt = ', g, '}'];
                                         end
% ---------------------------------------- %
if isfield(DATA, 'titleFsz')
       TitleFsz = DATA.titleFsz;
else;  TitleFsz = 14;                    end
% ---------------------------------------- %
if isfield(DATA, 'xlabel ')
       FigXlabl = DATA.xlabel ;
else;  FigXlabl = '{\bf u(t)}';          end
% ---------------------------------------- %
if isfield(DATA, 'xlablFsz')
       xlablFsz = DATA.xlablFsz;
else;  xlablFsz = 16;                    end
% ---------------------------------------- %
if isfield(DATA, 'ylabel ')
       FigYlabl = DATA.ylabel ;
else;  FigYlabl = '{\bf v(t)}';          end
% ---------------------------------------- %
if isfield(DATA, 'ylablFsz')
       ylablFsz = DATA.ylablFsz;
else;  ylablFsz = 16;                    end
% ---------------------------------------- %
if isfield(DATA, 'rotation')
       YlablRot = DATA.rotation;
else;  YlablRot = 0;                     end
% ---------------------------------------- %
if isfield(DATA, 'axesLineWidth')
         AxesLW = DATA.axesLineWidth;
else;    AxesLW = 2;                     end
% ---------------------------------------- %
if isfield(DATA, 'axesFontWeight')
         AxesFW = DATA.axesFontWeight;
else;    AxesFW = 'demi';                end
% ---------------------------------------- %
if isfield(DATA, 'axesFontSize')
         AxesFS = DATA.axesFontSize;
else;    AxesFS = 16;                    end
% ---------------------------------------- %
%
options = odeset('RelTol', 10*tol, 'AbsTol', tol*[0.01 0.01]);
%
umin = ejes(1); umax = ejes(2); vmin = ejes(3); vmax = ejes(4);
arrow = (sqrt((umax-umin)^2 + (vmax-vmin)^2))./FAR; % Length of arrow to use.
% Rmax2 ....................... switch radius squared: turn-off nonlinearity
%                               to avoid finite time blow-up in solution.
Rmax2 = 1.2*(max(umin^2, umax^2) + max(vmin^2, vmax^2));
%
% **** Now code starts. ****************************************************
%
% +++ Set up figure ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
fhndl = figure;
if isfield(DATA, 'FigName')
       FigName = DATA.FigName;
else;  FigName = ['  PHPL_plot_v2 (Fig. ', num2str(fhndl), '#)  '];  end
set(fhndl, 'Name', FigName, 'NumberTitle', 'off')
whitebg(FBG)
axis(ejes)
hold on
grid on
if isempty(FN)
   title( FigTitle, 'FontSize', TitleFsz)
   xlabel(FigXlabl, 'FontSize', xlablFsz) 
   ylabel(FigYlabl, 'FontSize', ylablFsz, 'Rotation', YlablRot)
else
   title( FigTitle, 'FontName', FN, 'FontSize', TitleFsz)
   xlabel(FigXlabl, 'FontName', FN, 'FontSize', xlablFsz) 
   ylabel(FigYlabl, 'FontName', FN, 'FontSize', ylablFsz, ...
                    'Rotation', YlablRot)
end
% --- Doing this after the titles assures it does not affect them.
%     Else get the FontWeight, etc. screwing up the title.
set(gca, 'FontUnits', 'points', 'FontWeight', AxesFW, ...
         'LineWidth', AxesLW, 'FontSize', AxesFS)
drawnow
%
for n=1:length(U);
    glo_dir = 1;
    t_0 = 0; y_0 = [U(n); V(n)];
    [T, Y] = ode45(fun, [t_0, t_f(n)], y_0, options);
    %
    % --- MAKE NOW A SMALL ARROW IN THE DIRECTION OF THE FLOW.
    %
    u0 = y_0(1);
    v0 = y_0(2);
    eval(['y_1 = ', fun, '(t_0, y_0);']);
    Du = y_1(1);
    Dv = y_1(2);
    step = sqrt(Du^2+Dv^2);
       u1 = u0 + (arrow(n)/step)*Du;
       v1 = v0 + (arrow(n)/step)*Dv;
       u2 = u0 + (arrow(n)/step)*Dv/4;
       v2 = v0 - (arrow(n)/step)*Du/4;
       u3 = u0 - (arrow(n)/step)*Dv/4;
       v3 = v0 + (arrow(n)/step)*Du/4;
    fill([u1, u2, u3], [v1, v2, v3], COL(n, :))
    plot(Y(:,1), Y(:,2), LS(n,:), 'Color', COL(n,:), 'LineWidth', LWn(n))
    %
    if BF == 1
       glo_dir = -1;
       [T, Y] = ode45(fun, [t_0, t_f(n)], y_0, options);
       plot(Y(:,1), Y(:,2), LS(n,:), 'Color', COL(n,:), 'LineWidth', LWn(n))
    end
    drawnow
end
hold off
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y_dot = PHPLplotFUNv2(t, y)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function y_dot = PHPLplotFUNv2(t, y)
%
% Auxiliary function used to evaluate ODE right hand side.
% Note the trick used to avoid blowup (the flow field is driven to zero at the
% radius Rmax = 2*Rmax2):
%                       p(x) = (2*x-1)*(x-2)^2 satisfies: p(1) = 1;  p(2) = 0;
%                                                        p'(1) = 0; p'(2) = 0;
% so that it can be used to "turn-off" the equation "smoothly".
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global glo_f  glo_g  glo_dir  Rmax2
%
u = y(1); v = y(2);
y_dot = y;
y_dot(1) = glo_dir*eval(glo_f);
y_dot(2) = glo_dir*eval(glo_g);
%
y2 = sum(y.^2);
if y2 > Rmax2
   p = y2/Rmax2;
   y_dot = ((2*p-1)*(p-2)^2)*y_dot;
   if p > 2
      y_dot = 0;
   end
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% COMPACT LISTING OF INPUTS
% f
% g
% DATA.U ........ DATA.U = [u10, u20, ... un0, ... uN0].
% DATA.V ........ DATA.U = [v10, v20, ... vn0, ... vN0].
% DATA.axis ..... DATA.axis = [umin umax vmin vmax].
%
% DATA.t_f ...... DEFAULT: DATA.t_f(n) = 25 for all n=1:N.
% DATA.BackForw . OPTIONS: 0 or 1.
%                 DEFAULT: DATA.BackForw = 1.
% DATA.COLORS ... e.g.: DATA.COLORS = ['r'; 'b'; ...];
%                       DATA.COLORS = [1 0 0; 0 0 1; ...];
%                 DEFAULT: all curves plotted in BLACK.
% DATA.LineStyle  e.g. DATA.LineStyle = [' -';  '--';  '-.';  ' :'; ... ].
%                 DEFAULT: all curves plotted in SOLID LINE '-'.
% DATA.LineWidth  Column array giving the LineWidth for each orbit.
%                 DEFAULT: all curves plotted with LineWidth = 2.
% DATA.FracArrow  Dir. arrow length = (diagonal plot. rectangle)/FracArrow.
%                 DEFAULT: DATA.FracArrow(n) = 25 for n=1:N.
% DATA.tol ...... DEFAULT: DATA.tol = 1e-7.
% DATA.fun ...... DEFAULT: DATA.fun = 'PHPLplot_v2'.
% DATA.FigName .. DEFAULT: 'PHPL_plot_v2 (Fig. #)'  (# is the figure handle).
% DATA.FontName . FontName for title, xlabel and y label of the figure.
%                 DEFAULT: MatLab's default (depends on OS).
% DATA.title .... DEFAULT: '{\bf du/dt = f and dv/dt = g}'
% DATA.titleFsz . FontSize for title. ................ DEFAULT: 14.
% DATA.xlabel ... String with the figure xlabel. ..... DEFAULT: '{\bf u(t)}'.
% DATA.xlablFsz . FontSize for xlabel. ............... DEFAULT: 16.
% DATA.ylabel ... String with the figure ylabel. ..... DEFAULT: '{\bf v(t)}'.
% DATA.ylablFsz . FontSize for ylabel. ............... DEFAULT: 16.
% DATA.rotation . Orientation of ylabel. In degrees,
%                 counter-clock-wise from horizontal.  DEFAULT:  0.
% DATA.axesLineWidth ................................. DEFAULT:  2.
% DATA.axesFontWeight ................................ DEFAULT: 'demi'.
% DATA.axesFontSize .................................. DEFAULT: 16.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version a by Rodolfo R. Rosales  10-25-1999.
% Version b by Rodolfo R. Rosales  10-18-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
