function [O1, O2, O3] = PHPLvdpFUN(t, Y, flag)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [O1, O2, O3] = PHPLvdpFUN(t, Y, flag)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).
% MIT, Spring 2000 - Fall 2004. R. R. Rosales.
%
% Function to evaluate the van der Pol right hand side, in the way required by
% ode15s and odeset (both part of MATLAB version 5.x):
%
% Y   = [  x;   y]  \  x_t = y;             / The parameter L > 0 is given by
% Y_t = [x_t; y_t]  /  y_t = L*(1-x^2)*y-x; \ the global variable: gPHPLvdpL.
%                                                 ^^^^^^^^^^^^^^^^^^^^^^^^^^
% When nargin = 2 or flag is empty: O1 = Y_t, O2 and O3 not assigned.
% If flag = 'events':   O1 = x - Thus ode15s detects zero crossings of x.
%                       O2 = 1 - Thus ode15s stops at a zero crossing.
%                       O3 = 1 - Detect zero crossing with x increasing only.
% If flag = 'jacobian': O1 = DF/DY.
%
% VECTORIZED:   [Y1_t(t), Y2_t(t), ...] = PHPLvdpFUN(t, [Y1, Y2, ...]).
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global gPHPLvdpL
%
if nargin < 3 | isempty(flag) % --- Return Y_t on O1.
   O1 = [ Y(2, :); ...
          gPHPLvdpL*(1 - Y(1, :).^2).*Y(2, :) - Y(1, :)];
else % ---------------------------- Return flag options in O1, O2 and O3.
   switch(flag)
      case 'events'
         O1 = Y(1); % --- detect zero crossings of x.
         O2 = 1;    % --- zero crossing stops calculation.
         O3 = 1;    % --- detect only a zero crossing with x increasing.
      case 'jacobian'
         O1 = [ 0, 1; -2*gPHPLvdpL*Y(1)*Y(2) - 1, gPHPLvdpL*(1-Y(1)^2)];
      otherwise
         error('Unknown flag')
   end
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  04-25-2000.
% Version 02 by Rodolfo R. Rosales  10-20-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
