function [Per, Y0, Er, T, Y] = PHPLvdpLiC(L, Tol)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Per, Y0, Er, T, Y] = PHPLvdpLiC(L, Tol)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).
% MIT, Spring 2000 - Fall 2004. R. R. Rosales.
%
% Evaluates the limit cycle for the van der Pol oscillator:
%
% x_t = y,
% y_t = L*(1-x^2)*y - x,
%
% as a function of the parameter L > 0. Calls PHPLvdPFUN, ode15s and odeset.
%                                       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% Tol ... Tolerance (absolute) in computing Y0 (and, by implication, in Per).
%         Optional (default = 1e-5).
% Per ... Period of the limit cycle.
% Y0 .... Point [x; y] = [0; y0] --- with y0 > 0 --- on limit cycle.
% T  .... Column vector of times. Let Nt = length(T), then T(0)=0 & T(Nt)=Per.
% Y  .... Nt by 2 array with the limit cycle.  Each row corresponds to a time:
%         Y(n, :) = [x(T(n)), y(T(n))], with Y(0) = Y(Nt) = Y0'.
%                                            ^^^^^^^^^^^^^^^^^^ within Tol.
% Er  ... Error returning to Y0, when solving the equation for one period=Per.
%
% WARNING: Before trusting any output (especially for L large) compute it with
%          two tolerances (at least) and compare the outputs.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global gPHPLvdpL
if nargin < 2; Tol = 1e-5; end
%
gPHPLvdpL = L; % --- needed by PHPLvdPFUN.
%
opciones = odeset('Abstol',  Tol*1e-5, ...
                  'RelTol',  Tol*1e-2, ...
                  'Events',      'on', ...
                  'Jacobian',    'on', ...
                  'Vectorized',  'on');
tspan = [0; 10+100*L]; % --- Chose very large time, much larger than period.
Y0    = [-2; 0]; % --- Initial point almost on limit cycle, with angle = pi.
%                      The limit cycle is tracked counter-clock-wise, so the
%                      1-st stop of the integration occurs at angle = pi/2.
%                      Note that ode15s is set up to stop when x=0, x_t > 0.
GO    = 1;
%
while GO == 1
   [T, Y, TE, YE, IE] = ode15s('PHPLvdPFUN', tspan, Y0, opciones);
   Y1 = YE';
   Er = norm(Y1-Y0);
   if Er < Tol; GO = 0; end
   Y0 = Y1;
   Y0(1) = 100*eps; % -- This is so ode15s does not stop right away.
end
Per = TE;
Y0  = YE';
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  04-25-2000.
% Version 02 by Rodolfo R. Rosales  10-20-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
