function [Per, y0] = PHPLvdpPER(L)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Per, y0] = PHPLvdpPER(L)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).
% MIT, Spring 2000 - Fall 2004. R. R. Rosales.
%
% Evaluates the limit cycle period (and "initial data" y0) for the van der Pol
% oscillator:
%              x_t = y,   y_t = L*(1-x^2)*y - x,
%
% as a function of the parameter L > 0.  The value of y0 is defined by: y0 > 0
% and (0, y0) is in the limit cycle. L can be an array.
%                                    ^^^^^^^^^^^^^^^^^^
% Uses the asymptotic formula for L large and an interpolant for other values.
%
% Per = period ...... Array of the same size as L. Relative Error < 1.5e-4
% y0 ................ Array of the same size as L. Relative Error < 0.8e-4
%
% Copyright at the end of this script file.
%  
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% NOTE: in oder to calculate the interpolants I used the data for the period &
%       value y0 stored in the directory: Phase_Plane:Private.
%       See: PHPLvdpPER_TEST, SAVEperiodVDP, vdP_period_intp and
%            CHECKvdP_period (in particular this last one, since
%            the stored data is not very accurate L very large.
%
% --- 0 <= L <= 3 (RelError(Per) < 3.0e-5; RelError(y0) < 3.1e-5).
%
a = [6.28304897470264; ...
     0.39430928954566; ...
    -0.01301063354241; ...
    -0.00139739321575; ...
     0.00040063773856; ...
    -0.00003755992784; ...
     0.00000133285820];
Per1 = a(1) + a(2)*(L.^2 )./(1+(0.10*L).^4 ) + ...
              a(3)*(L.^4 )./(1+(0.10*L).^6 ) + ...
              a(4)*(L.^6 )./(1+(0.25*L).^8 ) + ...
              a(5)*(L.^8 )./(1+(0.25*L).^10) + ...
              a(6)*(L.^10)./(1+(0.25*L).^12) + ...
              a(7)*(L.^12)./(1+(0.25*L).^14);
a = [1.99996019351510; ...
     0.17756344698396; ...
    -0.00366427891506; ...
    -0.00142833213205; ...
     0.00031938098770; ...
    -0.00002958537660; ...
     0.00000102702751];
y0_1 = a(1) + a(2)*(L.^2 )./(1+(0.10*L).^4 ) + ...
              a(3)*(L.^4 )./(1+(0.10*L).^6 ) + ...
              a(4)*(L.^6 )./(1+(0.25*L).^8 ) + ...
              a(5)*(L.^8 )./(1+(0.25*L).^10) + ...
              a(6)*(L.^10)./(1+(0.25*L).^12) + ...
              a(7)*(L.^12)./(1+(0.25*L).^14);
%
% --- 3 <= L <= 30 (RelError(Per) < 3.0e-5 ; RelError(y0) < 2.9e-5).
%
a = [1.12388480988519; ...
     1.58821017341919; ...
     2.63410479740640; ...
     0.00045457124472; ...
     0.00000022373736; ...
     0.00000001359801; ...
    -0.00000507480923];
Per2 = a(1) + a(2)*L + ...
       a(3)*(L.^4)./(1+(0.89*L).^(13/3)) + ...
       a(4)*(L.^2)./(1+(0.020*L).^4) + ...
       a(5)*(L.^4)./(1+(0.060*L).^6) + ...
       a(6)*(L.^6)./(1+(0.120*L).^8) + ...
       a(7)*(L.^8)./(1+(0.331*L).^10);   
a = [0.41053320178926; ...
     0.66075156072498; ...
     0.99279605292385; ...
     0.00008656841532; ...
     0.00000001743898; ...
     0.00000000522230; ...
    -0.00000063673924];   
y0_2 = a(1) + a(2)*L + ...
       a(3)*(L.^4)./(1+(0.97*L).^(13/3)) + ...
       a(4)*(L.^2)./(1+(0.020*L).^4) + ...
       a(5)*(L.^4)./(1+(0.059*L).^6) + ...
       a(6)*(L.^6)./(1+(0.119*L).^8) + ...
       a(7)*(L.^8)./(1+(0.310*L).^10);
%
% --- 30 <= L <= 400 (RelError(Per) < 8e-5 ; RelError(y0) < 8e-5).
%
a = [0.16728977836106; ...
     1.61350134892424; ...
     6.14002688312639; ...
    -0.00000068449917];
Per3 = a(1) + a(2)*L + a(3)*(L+eps).^(-1/3) + ...
       a(4)*(L.^2)./(1+(0.01*L).^4);
a = [0.12472122971397; ...
     0.66650186629093; ...
     1.70973434785075; ...
    -0.00000084344617];
y0_3 = a(1) + a(2)*L + a(3)*(L+eps).^(-1/3) + ...
       a(4)*(L.^2)./(1+(0.01*L).^4);   
%
% --- Asymptotic formulas, valid for large values of L.
%     Note: the true asymptotic formula has 2*2.33810741... instead of 2*pi
%           in the coefficient for L^(-1/3).  But 2*pi seems to work better
%           in the range 400<L<10000 (and for L>10000 the relative error is
%           smaller than the given limits). In the same fashion, 9/4 is not
%           the true asymptotic coefficient (it is just a fit to the data).
%     Note:  although the absolute errors of the data in vdP_period.mat are
%            not so hot for L large; the relative errors appear OK. See the
%            help for CHECKvdP_period.
%
coef = (3 - 2*log(2));
PerA = coef*L + (2*pi)*(L+eps).^(-1/3);% --- RelError < 1.4e-4 for L > 400
y0_A = (2/3)*L + (9/4)*(L+eps).^(-1/3);% --- RelError < 1.1e-5 for L > 400
%

%
Per = ( 0 <= L).*(L <   3).*Per1 + ...
      ( 3 <= L).*(L <  30).*Per2 + ...
      (30 <= L).*(L < 400).*Per3 + (400 <= L).*PerA;
y0  = ( 0 <= L).*(L <   3).*y0_1 + ...
      ( 3 <= L).*(L <  30).*y0_2 + ...
      (30 <= L).*(L < 400).*y0_3 + (400 <= L).*y0_A;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  04-25-2000.
% Version 02 by Rodolfo R. Rosales  10-20-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF

