function [Pper, Eap, Erp, Py0, Eay, Ery] = PHPLvdpPOL(Lmin, Lmax, Nd, tol)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Pper, Eap, Erp, Py0, Eay, Ery] = PHPLvdpPOL(Lmin, Lmax, Nd, tol)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).
% MIT, Spring 2000 - Fall 2004. R. R. Rosales.
%
% Pper ... Coefficients of an Nd-degree polynomial approximating the period of
%          the van der Pol oscillator limit cycle, for Lmin < L < Lmax.
%          x_t = y,
%          y_t = L*(1-x^2)*y - x.
%          Set up to be evaluated by polyval.
% tol .... Error (absolute) tolerance used when calling PHPLvdpLiC.
%          Optional; default = 1e-5.
% Eap .... L_infinity norm of the error in the approximation to the period.
% Erp .... Maximum relative error in the approximation to the period.
%
% Consider the point x=0 and y=y0 > 0 on the limit cycle. Then:
%
% Py0 .... Coefficients of an Nd-degree polynomial approximating y0.
%          Set up to be evaluated by polyval.
% Eay .... L_infinity norm of the error in the approximation to y0.
% Ery .... Maximum relative error in the approximation to y0.
%
% Calls: PHPLvdpLiC, polyfit and polyval.
%        ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% Uses PHPLvdpLiC to get the basic data needed for polynomial interpolation.
% As it computes it displays:  the computed L's and the corresponding values
% for the period and y0, with an estimate of the error in y0.
%
% When it finishes, it displays: the maximum interpolation error in both the
% period and in y0.   It also plots the calculated quantities and the errors
% by the interpolations (the figures have zoom and grid turned on).
%
% WARNING: For large values of L the vdP equation is very stiff.    Then the
%          results may not be very accurate.   Compare the output of running
%          the script with at least two tolerances before trusting it.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = Lmin + (Lmax-Lmin)*(0:10*Nd)/(10*Nd);% --- L's to interpolate from.
if nargin < 4; tol = 1e-5; end
Tol = tol;
for n=1:10*Nd+1
   [Per(n), Y0, Er] = PHPLvdpLiC(L(n), Tol);
   y0(n) = Y0(2);
   disp([' L = ', num2str(L(n), '%011.4e\t'), ' Period = ', ...
         num2str(Per(n), '%011.4e\t'), ' y0 = ', ...
         num2str(y0(n),  '%011.4e\t'), ' Error = ', ...
         num2str(Er,      '%08.1e\t')])
end
%
figure
whitebg('k')
Pper = polyfit(L, Per, Nd);
Y = polyval(Pper, L);
E = Per-Y;
fprintf('\n')
Eap = max(abs(E));
Erp = max(abs(E./Per));
disp([' Period max absolute error = ', num2str( Eap, '%011.4e')])
disp([' Period max relative error = ', num2str( Erp, '%011.4e')])
fprintf('\n')
plot(L, Per, '-c', 'LineWidth', 2)
whitebg('k')
hold on
plot(L, E, '--r', 'LineWidth', 2)
grid on
zoom on
title(['Period in blue; error in red.  Degree: Nd = ', num2str(Nd)], ...
       'FontSize', 16, 'FontName', 'Bold')
xlabel(['L     (Errors: Rel = ', num2str(Erp, '%7.1e'), ...
        ',  Abs = ', num2str(Eap, '%7.1e'), ')'], ...
       'FontSize', 16, 'FontName', 'Bold')
ylabel('Period and error.', 'FontSize', 16, 'FontName', 'Bold')
set(gca, 'FontSize', 12, 'FontWeight', 'bold')
%
figure
whitebg('k')
Py0 = polyfit(L, y0, Nd);
Y = polyval(Py0, L);
E = y0-Y;
Eay = max(abs(E));
Ery = max(abs(E./y0));
disp([' y0 max absolute error = ', num2str( Eay, '%011.4e')])
disp([' y0 max relative error = ', num2str( Ery, '%011.4e')])
fprintf('\n')
disp(' Zoom is on in both figures.')
fprintf('\n')
plot(L, y0, '-c', 'LineWidth', 2)
whitebg('k')
hold on
plot(L, E, '--r', 'LineWidth', 2)
grid on
zoom on
title(['y0 in blue; error in red.  Degree: Nd = ', num2str(Nd)], ...
       'FontSize', 16, 'FontName', 'Bold')
xlabel(['L     (Errors: Rel = ', num2str(Ery, '%7.1e'), ...
        ',  Abs = ', num2str(Eay, '%7.1e'), ')'], ...
       'FontSize', 16, 'FontName', 'Bold')
ylabel('y0 and error.', 'FontSize', 16, 'FontName', 'Bold')
set(gca, 'FontSize', 12, 'FontWeight', 'bold')
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  04-25-2000.
% Version 02 by Rodolfo R. Rosales  10-20-2004.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
