function [X, Y] = LowConvHull(x0, y0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [X, Y] = LowConvHull(x0, y0)
%
% 18.385 (Nonlinear Dynamical Systems). MIT, Fall 2000-2005. R. R. Rosales.
% _________________________________________________________________________
% Find the LOWER CONVEX HULL of a set of points in the plane:
%
% Let x0 and y0 be two arrays (of ARBITRARY but EQUAL SIZES) with the x and
% y cartesian coordinates of a set S of points in the plane. Then X & Y are
% (equal length) COLUMN vectors,  with the coordinates of the lower side of
% the boundary of the convex hull of S.
%
% The upper boundary of the convex hull (thus the whole convex hull) can be
% found using LowConvHull(x0, -y0).  Note that the first and last points in
% the upper and lower convex hulls coincide.
%
% NOTE 1: X is sorted, with X(n) < X(n+1) for any n=2:N-2, N=length(X).
%         It is X(1) = X(2) (resp.: X(N-1) = X(N)) when the convex hull has
%         a vertical segment on its left side (resp.:  when the convex hull
%         has a vertical segment on its right side.)
%
% NOTE 2: MatLab has the script convhull, to calculate the convex hull of a
%         set in the plane; but it seems to be slower than the simple algo-
%         rithm used here (even though it is built-in!)
%
% Algorithm
%       NOTATION: below N = length(X) = length(Y) --- actualized as X and Y
%                 change.
%                 P(n) = (X(n), Y(n)) is a point in the plane.
%
% 1) Order the set of points by increasing x and place them in the 2 column
%    arrays X and Y (thus X is nondecreasing.) Eliminate repeated points.
%
% 2) If X(1) = X(2) = ... = X(M) -- for some M > 1 -- reorder the points so
%    that Y(1) = max(Y(1), ... Y(M))  and  Y(2) = min(Y(1), ... Y(M)), with
%    all the other points gone.
%    This guarantees P(1) is always in the hull -- when X(1) < X(2) we have
%    to do nothing to guarantee this.
%
% 3) If X(N-M) = X(N-M+1) = ... = X(N) -- for some M > 0 -- reorder X and Y
%    so Y(N) = max(Y(N-M), ... Y(N)) & Y(N-1) = min(Y(N-M), ... Y(N)), with
%    all the other points gone.
%    This guarantees P(N) is always in the hull -- when X(N - 1) < X(N)  we
%    have to do nothing to guarantee this.
%
% 4) Consider the point P(n), for n=2:N-1. Then, if X(n) = X(m) (m = n + 1,
%    or m = n - 1) and Y(n) > Y(m), we can eliminate the point from consid-
%    eration as a member of the lower boundary of the convex hull.   Elimi-
%    nate all points satisfying this condition.
%    --- Note: we cannot have Y(n) = Y(m), since the points are distinct.
%    --- After this step, we know that x(n) < x(n+1) for n=2:N-2,  since no
%        more than one value of y will be kept for each value of X(1) < x <
%        X(N) in the set of points considered.
%
% Repeat (5) below till no points are eliminated. What remains is the lower
% convex hull.    This process recursively eliminates all points not in the
% hull. Since the "end points" are in it, we do not worry about them.
%
% 5) Consider now the points P(n), for n=2:N-1. Find all that are above the
%    line segment from P(n-1) to P(n+1), and eliminate them.
%    The condition of being above the line segment is:
%
%    (X(n+1) - X(n-1))*Y(n) >= Y(n+1)*(X(n)-X(n-1)) + Y(n-1)*(X(n+1)-X(n)).
%
%    --- Notice that the prior steps (1--4) ensure X(n+1) - X(n-1) > 0.
%
% Copyright at the end of this script file.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if min(size(x0) == size(y0)) == 0
   fprintf('\n Error in LowConvHull.')
   fprintf('\n >>>>>>>>>>>>>>>>>>>>> x0 and y0 must be the same size. \n')
   return
end
%
N = prod(size(x0));
%
if N == 0
   fprintf('\n Error in LowConvHull.')
   fprintf('\n >>>>>>>>>>>>>>>>>>>>> x0 and y0 are empty arrays. \n')
   return
end
%
if ~(isnumeric(x0) & isreal(x0))
   fprintf('\n Error in LowConvHull.')
   fprintf('\n >>>>>>>>>>>>>>>>>>>>> x0 not a real valued array. \n')
   return
end
%
if ~(isnumeric(y0) & isreal(y0))
   fprintf('\n Error in LowConvHull.')
   fprintf('\n >>>>>>>>>>>>>>>>>>>>> y0 not a real valued array. \n')
   return
end
%
% _________________________________________________________________________
% 1) Order the set of points by increasing x and place them in the 2 column
%    arrays X and Y (thus X is nondecreasing.) Eliminate repeated points.
%
X = reshape(x0, N, 1); [X, J] = sort(X);
Y = reshape(y0, N, 1);      Y = Y(J);
%
if N == 1;   return;   end
%
I = 1:N-1;   J = find( (X(I) == X(I+1)) & (Y(I) == Y(I+1)) );
if ~isempty(J);   X(J) = [];   Y(J) = [];   N = length(X);   end
%
if N == 1;   return;   end
%
% _________________________________________________________________________
% 2) If X(1) = X(2) = ... = X(M) -- for some M > 1 -- reorder the points so
%    that Y(1) = max(Y(1), ... Y(M))  and  Y(2) = min(Y(1), ... Y(M)), with
%    all the other points gone.
%    This guarantees P(1) is always in the hull -- when X(1) < X(2) we have
%    to do nothing to guarantee this.
%
%    -- Because X is ordered, J = 1:M below.
%    -- Since there are no repeated points, y1 > y2 below.
%
J = find(X == X(1)); M = length(J);
if M > 1
   y1 = max(Y(J));   y2 = min(Y(J));   Y(1) = y1;   Y(2) = y2;
   X(3:M) = [];   Y(3:M) = [];   N = length(X);
end
%
if N == 2;   return;   end
%
% _________________________________________________________________________
% 3) If X(N-M) = X(N-M+1) = ... = X(N) -- for some M > 0 -- reorder X and Y
%    so Y(N) = max(Y(N-M), ... Y(N)) & Y(N-1) = min(Y(N-M), ... Y(N)), with
%    all the other points gone.
%    This guarantees P(N) is always in the hull -- when X(N - 1) < X(N)  we
%    have to do nothing to guarantee this.
%
%    -- Because X is ordered, J = N-M+1:N below.
%    -- Since there are no repeated points, y1 > y2 below.
%
J = find(X == X(N)); M = length(J);
if M > 1
   y1 = max(Y(J));   y2 = min(Y(J));   Y(N-1) = y2;   Y(N) = y1;
   X(N-M+1:N-2) = [];   Y(N-M+1:N-2) = [];   N = length(X);
end
%
% _________________________________________________________________________
% 4) Consider the point P(n), for n=2:N-1. Then, if X(n) = X(m) (m = n + 1,
%    or m = n - 1) and Y(n) > Y(m), we can eliminate the point from consid-
%    eration as a member of the lower boundary of the convex hull.   Elimi-
%    nate all points satisfying this condition.
%    --- Note: we cannot have Y(n) = Y(m), since the points are distinct.
%    --- After this step, we know that x(n) < x(n+1) for n=2:N-2,  since no
%        more than one value of y will be kept for each value of X(1) < x <
%        X(N) in the set of points considered.
%
I = 2:N-1;
J = find( (X(I)==X(I+1) & Y(I)>Y(I+1)) | (X(I)==X(I-1) & Y(I)>Y(I-1)) );
if ~isempty(J);   X(J+1) = [];   Y(J+1) = [];   end
%
% _________________________________________________________________________
% Repeat (5) below till no points are eliminated. What remains is the lower
% convex hull.    This process recursively eliminates all points not in the
% hull. Since the "end points" are in it, we do not worry about them.
%
% 5) Consider now the points P(n), for n=2:N-1. Find all that are above the
%    line segment from P(n-1) to P(n+1), and eliminate them.
%    The condition of being above the line segment is:
%
%    (X(n+1) - X(n-1))*Y(n) >= Y(n+1)*(X(n)-X(n-1)) + Y(n-1)*(X(n+1)-X(n)).
%
%    --- Notice that the prior steps (1--4) ensure X(n+1) - X(n-1) > 0.
%
GO = 1;
while GO == 1
   I = 2:length(X)-1;
   x1 = X(I-1);  x2 = X(I);  x3 = X(I+1);
   y1 = Y(I-1);  y2 = Y(I);  y3 = Y(I+1);
   J  = find((x3-x1).*y2 >= y1.*(x3-x2)+y3.*(x2-x1));
   if ~isempty(J);   X(J+1) = [];   Y(J+1) = [];   GO = 1;
   else; GO = 0;
   end
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  09-24-2000.
% Version 02 by Rodolfo R. Rosales  12-10-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
