function LowConvHullDemo(N, FLAG)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function LowConvHullDemo(N, FLAG)
%
% 18.385 (Nonlinear Dynamical Systems). MIT, Fall 2000-2005. R. R. Rosales.
% _________________________________________________________________________
% Function to demonstrate and test the script LowConvHull.
%
% Picks random points in the square 0.1 <= x, y <= 0.9 and finds the convex
% hull. The Lower Convex hull is plotted in cyan,  the Upper Convex Hull in
% magenta and the points in the set in red with a yellow outline.
%
% Both N and FLAG are optional. The defaults are N = 25 and FLAG = 1.
% Use N = [] as a place-holder to skip it.
%
% FLAG = 1 Choose N random points in square.
%          Put them in column vector arrays.
% FLAG = 2 Choose N random points in square.
%          Put them in row vector arrays.
% FLAG = 3 Choose N^2 random points in square.
%          Put them in N by N arrays.
% FLAG = 4 Choose N random points in square.
%          Put them in N by 2 arrays, where the columns repeat.
% FLAG = 5 Choose N random points in square.
%          Put them in N by 2 arrays, where the columns repeat for x and
%          for y the second column is half-way the distance from y = 0.9
%          of the first column.
% FLAG = 6 N+1 points of the form y = 0.5 + 0.4*sigma*sin(pi*x),
%          where sigma = +/-1 (random) and x = 0.1 + 0.8*(0:N)/N.
% FLAG = 7 Similar to FLAG = 6,  but the x appear 4 times each,  the
%          amplitude of the sine oscillations is 0.2 instead of 0.4,
%          and a random perturbation of amplitude 0.2 is added to y.
%
% Copyright at the end of this script file.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SSz = get(0, 'ScreenSize');
SWt = SSz(3);% ---------- Screen Width
SHt = SSz(4);% ---------- Screen Height
Sml = min(SWt, SHt);% --- Screen minimum length.
Fwh = 0.65;  % ---------- Fig. Width = Height; as fraction of SmL.
POS = [0.02*SWt, 0.94*SHt-Fwh*Sml, Fwh*Sml, Fwh*Sml];
figure;
set(gcf, 'Position',    POS)
whitebg('k')
hold on
grid on
zoom on
axis([0 1 0 1])
set(gca, 'FontUnits',  'points', ...
         'FontWeight', 'demi', ...
         'FontSize',    16, ...
         'LineWidth',   2, ...
         'Xtick',       [0.1 0.3 0.5 0.7 0.9], ...
         'Ytick',       [0.1 0.3 0.5 0.7 0.9], ...
         'Box',         'on')
axis square
xlabel('{\bf x}', 'FontSize', 20)
ylabel('{\bf y  }', 'FontSize', 20, 'Rotation', 0)
title('{\bf Lower and Upper Convex Hull.}', 'FontSize', 20)
%
%
if nargin == 0;   N = 25; FLAG = 1; end
if nargin == 1;           FLAG = 1; end
if isempty(N);    N = 25;           end
if isempty(FLAG);         FLAG = 1; end
%
if     FLAG == 1
   x0 = 0.8*rand(N, 1) + 0.1;
   y0 = 0.8*rand(N, 1) + 0.1;
elseif FLAG == 2
   x0 = 0.8*rand(1, N) + 0.1;
   y0 = 0.8*rand(1, N) + 0.1;
elseif FLAG == 3
   x0 = 0.8*rand(N, N)+0.1;
   y0 = 0.8*rand(N, N)+0.1;
elseif FLAG == 4
   x0 = 0.8*rand(N, 1) + 0.1; x0 = [x0, x0];
   y0 = 0.8*rand(N, 1) + 0.1; y0 = [y0, y0];
elseif FLAG == 5
   x0 = 0.8*rand(N, 1) + 0.1; x0 = [x0, x0];
   y0 = 0.8*rand(N, 1) + 0.1; y0 = [y0, 0.5*(y0+0.9)];
elseif FLAG == 6
   x0 = 0.1 + 0.8*(0:N)/N;
   y0 = 0.5 + 0.4*sin(pi*x0).*sign(rand(1, N+1)-0.5);
elseif FLAG == 7
   x0 = 0.1 + 0.8*(0:N)/N; x0 = sort([x0, x0, x0, x0]);
   y0 = 0.5 + 0.2*sin(pi*x0).*sign(rand(1, 4*N+4)-0.5) ...
            + 0.4*(rand(1, 4*N+4)-0.5);
else
   fprintf('\n Incorrect FLAG value given.\n')
   return
end
Nx0_txt = num2str(prod(size(x0)));
%
tic;   [x, y] = LowConvHull(x0,  y0);   etime = toc;
   Nlh_txt = num2str(length(x));
   Tlh_txt = num2str(etime);
   plot(x,  y, '-c', 'LineWidth', 3)
%
tic;   [x, y] = LowConvHull(x0, -y0);   etime = toc;
   Nuh_txt = num2str(length(x));
   Tuh_txt = num2str(etime);
   plot(x, -y, '-m', 'LineWidth', 2)
%
plot(x0, y0, 'o', 'MarkerSize',  8, 'MarkerFaceColor', 'r', ...
                                    'MarkerEdgeColor', 'y')
%
fprintf('\n Done with LowConvHull; now use the MatLab thing. \n')
%
fprintf(['\n Number of points in Set .................. ', Nx0_txt])
fprintf(['\n Number of points in Lower Convex Hull .... ', Nlh_txt])
fprintf(['\n Number of points in Upper Convex Hull .... ', Nuh_txt])
fprintf('\n')
fprintf(['\n Seconds to compute Lower Convex Hull .......... ', Tlh_txt])
fprintf(['\n Seconds to compute Upper Convex Hull .......... ', Tuh_txt])
%
pause(1)
%
tic;  K = convhull(x0, y0);  etime = toc;  Tma_txt = num2str(etime);
fprintf(['\n Seconds to compute Hull (MatLab convhull) ..... ', Tma_txt])
fprintf('\n')
fprintf('\n To plot (on top) the MatLab computed hull, press any key.\n')
pause
plot(x0(K), y0(K), '-w', 'LineWidth', 3)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  09-24-2000.
% Version 02 by Rodolfo R. Rosales  12-10-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% EOF
