function [q, xc, h, y0] = PieceWiseParAppr(N, Y, X, x0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [q, xc, h, y0] = PieceWiseParAppr(N, Y, X, x0)
% _________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems). MIT, Fall 2000-2005. R. R. Rosales.
%
% Let Y and X be linear numerical (real) arrays of the same length, with
% a = min(X) and b = max(X).  Subdivide the interval [a, b] into N equal
% length intervals of width h = (b-a)/h.
%
% Let now P = P(x) be a C^1 piecewise parabolic function in [a b] (i.e.:
% a 2nd order polynomial in each sub-interval of length h and C^1 across
% the interval boundaries), such that:
%
%                L2 = Sum_{n=1:N} (Y(n) - P(X(n)))^2
% is minimized.
%
% This script provides such a function, in the form:
%
%         P(x) = Sum(n=1:N+2) q(n) Q((x-xc(n))/h),
%
% where xc(n) = a + (n-1.5)*h, the q(n) are some constants and Q is the
% elementary quadratic function defined in the subscript (below, see @#@)
% PiWiParElementFun.
% Furthermore:       y0 = P(x0), where x0 is a linear array with values
%                                in [a, b].
%
% q and xc are column vectors.
% The arguments y0 and x0 are optional.
%
% Copyright at the end of this script file.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if N <= 0; N = 1; end
N = ceil(N);
% --- Check X and Y linear arrays. Make them column.
%
S = size(X);
if     S(1) == 1; Nx = S(2); X = X';
elseif S(2) == 1; Nx = S(1);
else;  error(' X is not a linear array. '); end
%
S = size(Y);
if     S(1) == 1; Ny = S(2); Y = Y';
elseif S(2) == 1; Ny = S(1);
else;  error(' Y is not a linear array. '); end
%
% --- Check sizes
%
if Nx ~= Ny; error(' X and Y must have the same length. '); end
if Nx < N+2; error(' Not enough points for requested N. '); end
%
a  = min(X);
b  = max(X);
if b <= a; error(' Need min(X) < max(X). '); end
h  = (b-a)/N;
M  = N+2;
xc = (a + ((1:M)-1.5)*h); % Centers of intervals.
%
% --- Quantity to minimize can be written as: L2 = norm(Y - A*q, 2)
%     where A is the Nx by M array:
%
A  = PiWiParElementFun((X*ones(1, M) - ones(Nx, 1)*xc)/h);
q  = A\Y;
xc = xc';
%
if (nargin == 4) & (nargout == 4)
   y0 = 0;
   for n=1:M
      y0 = y0 + q(n)*PiWiParElementFun((x0-xc(n))/h);
   end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Q = PiWiParElementFun(x)
% @#@ %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function (x can be an array) computes the piecewise parabolic C^1
% function Q = Q(x) given by:
%
%  Q = 1 - (4/3)*r^2      if         r <= 0.5  ===> dQ/dr = - (8/3)*r
%    = (2/3)*(r - 1.5)^2  if  0.5 <= r <= 1.5  ===> dQ/dr = (4/3)*(r-1.5)
%    = 0                  if  1.5 <= r         ===> dQ/dr =  0
%
% where r = abs(x) and R = 1/3.
%
% Q is defined by:  i) Q is a 2nd degree polynomial (in x) on each of the
%                      3 intervals [-1.5, -0.5]; [-0.5, 0.5] & [0.5, 1.5].
%                  ii) Q vanishes for r >= 1.5
%                 iii) Q is C^1.
%                  iv) Q(0) = 1.
%
% Then, any C^1 piecewise parabolic function p(x) in a <= x <= b; defined
% on N equal intervals of length h = (b-a)/N
%      ________________________________________________________
%     | That is, p is a 2nd degree polynomial in each length h
%     | interval and is a C^1 function defined for x in [a, b];
%     | Note: this yields 3*N constants, restricted by 2*(N-1)
%     | equations. Thus N+2 free parameters.
%
% can be written as       p(x) = Sum(n=1:N+2) q_n Q((x-x_n)/h),
%
% where x_n = a + (n-1.5)*h, for some parameters q_n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r  = abs(x);
S1 = sign(r - 0.5);
S3 = sign(r - 1.5);
Q   = (1-S1).*(1-(4/3)*r.^2)/2 + (2/6)*(1-S1.*S3).*(r-1.5).^2;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  09-30-2000.
% Version 02 by Rodolfo R. Rosales  10-29-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% EOF
