function y0 = PieceWiseParEval(q, xc, h, x0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function y0 = PieceWiseParEval(q, xc, h, x0)
% _________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems). MIT, Fall 2000-2005. R. R. Rosales.
%
% If q, xc and h are as provided by PieceWiseParAppr, then:
%
%                          y0 = P(x0),
%
% where x0 is a linear array with values in [a, b], amd P is the piecewise
% parabolic approximation defined in PieceWiseParAppr.
%
% NOTE: actually, there is no need to restrict x0 to [a, b], though P is
%       compact support and vanishes for x < a-2*h and x > b + 2*h.
%
% Copyright at the end of this script file.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
y0 = 0;
for n=1:length(q)
      y0 = y0 + q(n)*PiWiParElementFun((x0-xc(n))/h);
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Q = PiWiParElementFun(x)
% @#@ %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function (x can be an array) computes the piecewise parabolic C^1
% function Q = Q(x) given by:
%
%  Q = 1 - (4/3)*r^2      if         r <= 0.5  ===> dQ/dr = - (8/3)*r
%    = (2/3)*(r - 1.5)^2  if  0.5 <= r <= 1.5  ===> dQ/dr = (4/3)*(r-1.5)
%    = 0                  if  1.5 <= r         ===> dQ/dr =  0
%
% where r = abs(x) and R = 1/3.
%
% Q is defined by:  i) Q is a 2nd degree polynomial (in x) on each of the
%                      3 intervals [-1.5, -0.5]; [-0.5, 0.5] & [0.5, 1.5].
%                  ii) Q vanishes for r >= 1.5
%                 iii) Q is C^1.
%                  iv) Q(0) = 1.
%
% Then, any C^1 piecewise parabolic function p(x) in a <= x <= b; defined
% on N equal intervals of length h = (b-a)/N
%      ________________________________________________________
%     | That is, p is a 2nd degree polynomial in each length h
%     | interval and is a C^1 function defined for x in [a, b];
%     | Note: this yields 3*N constants, restricted by 2*(N-1)
%     | equations. Thus N+2 free parameters.
%
% can be written as       p(x) = Sum(n=1:N+2) q_n Q((x-x_n)/h),
%
% where x_n = a + (n-1.5)*h, for some parameters q_n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r  = abs(x);
S1 = sign(r - 0.5);
S3 = sign(r - 1.5);
Q   = (1-S1).*(1-(4/3)*r.^2)/2 + (2/6)*(1-S1.*S3).*(r-1.5).^2;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  09-30-2000.
% Version 02 by Rodolfo R. Rosales  10-29-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF
