function [F, k] = fourierSER(f, t, PLOT)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [F, k] = fourierSER(f, t, PLOT)
% ____________________________________________________________________________
% 18.385 (Nonlinear Dynamical Systems).    MIT, Fall 1999-2005. R. R. Rosales.
%
% Let f=f(t) be a periodic function, with period T.
% Let N be some "large" natural number and let h=L/N.
% Then, in the INPUT variables we have:
%                                        t .... N vector: t(n) = (n-1)*h
%                                        f .... N vector: f(n) = f(t(n))
% where n=1:N (one period).
% The OUTPUT variables are vectors with (respectively) the Fourier coefficients
% and wavenumbers for f:
%                        f(t) = Sum{F(s)*exp(i*k(s)*t)},  <<=== sum over s.
%                        k(s) = 2*pi*s/L,
% where |s| <= (N-1)/2 if N is odd and |s| <= N/2 if N is even.
% ALGORITHM:
% Approximation of the Fourier coefficients by discrete sums yields:
%
%         F(s) = (1/L) Int_0^L f(t)*exp(-i*k(s)*t) dt =
%              = (1/N) Sum f(n)*w^(s*(n-1)),  where w=exp(-2*pi*i/N).
%                     n=1:N
%
% On the other hand:   fft(f)(j) = Sum f(n)*w^((j-1)*(n-1)), for j=1:N
%                                 n=1:N
%
% in the MatLab implementation of the fft. Since this formula is periodic in j
% of period j, we can use it to get the answer.
%
% The argument PLOT is optional:
%                  Anything there triggers a plot of abs(F) versus k.
%
% Copyright at the end of this script file.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (min(size(f)) ~= 1) | (min(size(t)) ~= 1)
   error('The input arrays must be vectors')
end
%
N = length(t);
%
if (length(f) ~= N) | (N < 2)
   error('The input arrays must be the same length N, with N > 1.')
end
%
h = (t(N)-t(1))/(N-1);
%
if h <= 0
   error('Must be t(N) - t(1) > 0.')
end
%
L = N*h;
%
M = floor(0.5*N+0.01);
k = (2*pi/L)*(-M:M);
G = (1/N)*fft(f);
%
if (abs(0.5*N-M) < 0.1) % --- Case N even:
   F = G([M+1:N, 1:M+1]);
else % ---------------------- Case N odd: 
   F = G([M+2:N, 1:M+1]);
end
%
FS  = 'FontSize';
FN  = 'FontName';
LW  = 'LineWidth';
HNB = 'HelveticaNarrowBold';
if nargin == 3
   figure
   whitebg('k')
   plot(k, abs(F), '-c', LW, 2)
   xlabel('Wave number k', FN, HNB, FS, 16)
   ylabel('Norm of Fourier coefficients.', FN, HNB, FS, 16)
   title([' N = ', num2str(N), ' and L = ', num2str(L)], FN, HNB, FS, 18)
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Massachusetts Institute of Technology
% Version 01 by Rodolfo R. Rosales  10-14-1999.
% Version 02 by Rodolfo R. Rosales  11-17-2005.
% 
% Permission is hereby granted, without payment, to copy this software
% and its documentation, if any,  for non-profit academic and research
% purposes,  provided that the above copyright notice, this paragraph,
% and the following three paragraphs appear in all copies of this
% software. Use of this software constitutes acceptance of these terms
% and conditions.
%
% IN NO EVENT SHALL MIT, OR THE AUTHOR, BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF MIT, OR THE AUTHOR, HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
%
% MIT, AND THE AUTHOR, SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED
% WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
% NON-INFRINGEMENT.
%
% THIS SOFTWARE IS PROVIDED "AS IS." MIT, OR THE AUTHOR, HAVE NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
% OR MODIFICATIONS.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF

